@@ -880,6 +880,39 @@ class Agent(object):
 				response["error"] = f"Key path [{path}] does not exist."
 				return response
 
+		# check agent connection.
+		output = utils.__execute__(["ssh-add", "-L"])
+		if "Failed to communicate" in output or "agent refused operation" in output or "Error connecting to agent" in output or "Connection refused" in output:
+			if reattempt:
+				try:
+					output = subprocess.check_output([f"ssh-add", "-D"])
+					print("delete output:",output)
+				except: a=1
+				try:
+					output = subprocess.check_output([f"ssh-add", "-k"])
+					print("kill output:",output)
+				except: a=1
+				try:
+					output = subprocess.check_output([f"eval", "`ssh-agent`"])
+					print("restart output:",output)
+				except: a=1
+				return self.add(
+					# the keys path.
+					path=path,
+					# the keys passphrase.
+					passphrase=passphrase,
+					# enable if you are using a smart card.
+					smart_card=smart_card,
+					# the smart cards pin code
+					pin=pin,
+					# default timeout (do not use).
+					timeout=timeout,
+					# reattempt (do not use).
+					reattempt=False,
+				)
+			else:
+				return r3sponse.error_response("Failed to communicate with the ssh-agent. Try logging out the current system user & logging back in (or execute [$ eval `ssh-agent`]).")
+
 		# with passphrase.
 		response = utils.__default_response__()
 		if smart_card or passphrase != None:
@@ -932,11 +965,11 @@ class Agent(object):
 					spawn.sendline(passphrase)
 
 			except pexpect.exceptions.TIMEOUT:
-				return r3sponse.error_response(f"Failed to add key [{path}] to the ssh agent #1.")
+				a=1
 			except pexpect.exceptions.EOF:
 				a=1
 			
-			# try output.
+			# handle output.
 			output = spawn.read().decode()
 
 			# check success.
@@ -944,7 +977,7 @@ class Agent(object):
 				return r3sponse.error_response("Provided an incorrect passphrase.")
 			elif "incorrect pin" in output.lower():
 				return r3sponse.error_response("Provided an incorrect pin code.")
-			elif "Failed to communicate" in output or "agent refused operation" in output:
+			elif "Failed to communicate" in output or "agent refused operation" in output or "Error connecting to agent" in output or "Connection refused" in output:
 				if reattempt:
 					try:
 						output = subprocess.check_output([f"ssh-add", "-D"])
@@ -993,7 +1026,7 @@ class Agent(object):
 			output = utils.__execute__(command=["ssh-add", path], shell=False, timeout=timeout)
 
 			# check success.
-			if "Failed to communicate" in output:
+			if "Failed to communicate" in output or "agent refused operation" in output or "Error connecting to agent" in output or "Connection refused" in output:
 				return r3sponse.error_response("Failed to communicate with the ssh-agent.")
 			elif "Identity added:" in output: 
 				return r3sponse.success_response(f"Successfully added key [{path}] to the ssh agent.")
